@@ -20,16 +20,20 @@ The Pygame documentation is pretty good, and can be found at http://www.pygame.o
 Unfortunately there is no sound with this game.
 """
 
+import os
 import sys
 import time
 import random
 import math
 import logging
+import logging.handlers
 
 import pygame
 import pygame.locals
 
+import constants
 import images
+from util import pygame_utils
 
 
 _moduleLogger = logging.getLogger(__name__)
@@ -80,36 +84,16 @@ def terminate():
     sys.exit()
 
 
-def makeSurfaceFromASCII(ascii, fgColor=(255, 255, 255), bgColor=(0, 0, 0)):
-    """Returns a new pygame.Surface object that has the image drawn on it as specified by the ascii parameter.
-    The first and last line in ascii are ignored. Otherwise, any X in ascii marks a pixel with the foreground color
-    and any other letter marks a pixel of the background color. The Surface object has a width of the widest line
-    in the ascii string, and is always rectangular."""
-
-    ascii = ascii.split('\n')[1:-1]
-    width = max([len(x) for x in ascii])
-    height = len(ascii)
-    surf = pygame.Surface((width, height))
-    surf.fill(bgColor)
-
-    pArr = pygame.PixelArray(surf)
-    for y in range(height):
-        for x in range(len(ascii[y])):
-            if ascii[y][x] == 'X':
-                pArr[x][y] = fgColor
-    return surf
-
-
-GOR_DOWN_SURF = makeSurfaceFromASCII(images.GOR_DOWN_ASCII, GOR_COLOR, SKY_COLOR)
-GOR_LEFT_SURF = makeSurfaceFromASCII(images.GOR_LEFT_ASCII, GOR_COLOR, SKY_COLOR)
-GOR_RIGHT_SURF = makeSurfaceFromASCII(images.GOR_RIGHT_ASCII, GOR_COLOR, SKY_COLOR)
-BAN_RIGHT_SURF = makeSurfaceFromASCII(images.BAN_RIGHT_ASCII, BAN_COLOR, SKY_COLOR)
-BAN_LEFT_SURF = makeSurfaceFromASCII(images.BAN_LEFT_ASCII, BAN_COLOR, SKY_COLOR)
-BAN_UP_SURF = makeSurfaceFromASCII(images.BAN_UP_ASCII, BAN_COLOR, SKY_COLOR)
-BAN_DOWN_SURF = makeSurfaceFromASCII(images.BAN_DOWN_ASCII, BAN_COLOR, SKY_COLOR)
-SUN_NORMAL_SURF = makeSurfaceFromASCII(images.SUN_NORMAL_ASCII, SUN_COLOR, SKY_COLOR)
-SUN_SHOCKED_SURF = makeSurfaceFromASCII(images.SUN_SHOCKED_ASCII, SUN_COLOR, SKY_COLOR)
-STAR_SURF = makeSurfaceFromASCII(images.STAR_ASCII, DARK_RED_COLOR, BLACK_COLOR)
+GOR_DOWN_SURF = pygame_utils.makeSurfaceFromASCII(images.GOR_DOWN_ASCII, GOR_COLOR, SKY_COLOR)
+GOR_LEFT_SURF = pygame_utils.makeSurfaceFromASCII(images.GOR_LEFT_ASCII, GOR_COLOR, SKY_COLOR)
+GOR_RIGHT_SURF = pygame_utils.makeSurfaceFromASCII(images.GOR_RIGHT_ASCII, GOR_COLOR, SKY_COLOR)
+BAN_RIGHT_SURF = pygame_utils.makeSurfaceFromASCII(images.BAN_RIGHT_ASCII, BAN_COLOR, SKY_COLOR)
+BAN_LEFT_SURF = pygame_utils.makeSurfaceFromASCII(images.BAN_LEFT_ASCII, BAN_COLOR, SKY_COLOR)
+BAN_UP_SURF = pygame_utils.makeSurfaceFromASCII(images.BAN_UP_ASCII, BAN_COLOR, SKY_COLOR)
+BAN_DOWN_SURF = pygame_utils.makeSurfaceFromASCII(images.BAN_DOWN_ASCII, BAN_COLOR, SKY_COLOR)
+SUN_NORMAL_SURF = pygame_utils.makeSurfaceFromASCII(images.SUN_NORMAL_ASCII, SUN_COLOR, SKY_COLOR)
+SUN_SHOCKED_SURF = pygame_utils.makeSurfaceFromASCII(images.SUN_SHOCKED_ASCII, SUN_COLOR, SKY_COLOR)
+STAR_SURF = pygame_utils.makeSurfaceFromASCII(images.STAR_ASCII, DARK_RED_COLOR, BLACK_COLOR)
 
 assert GOR_DOWN_SURF.get_size() == GOR_LEFT_SURF.get_size() == GOR_RIGHT_SURF.get_size()
 
@@ -124,26 +108,23 @@ def drawText(text, surfObj, x, y, fgcol, bgcol, pos='left'):
     If the pos parameter is "center", then the x,y parameter specifies the middle top point of the text rectangle."""
 
     textobj = GAME_FONT.render(text, 1, fgcol, bgcol) # creates the text in memory (it's not on a surface yet).
-    textrect = textobj.get_rect()
 
+    textrect = textobj.get_rect()
     if pos == 'left':
         textrect.topleft = (x, y)
     elif pos == 'center':
         textrect.midtop = (x, y)
+    elif pos == "right":
+        textrect.topright = (x, y)
+    else:
+        raise NotImplementedError("Unknown pos %r" % pos)
+
     surfObj.blit(textobj, textrect) # draws the text onto the surface
     """Remember that the text will only appear on the screen if you pass the pygame.Surface object that was
     returned from the call to pygame.display.set_mode(), and only after pygame.display.update() is called."""
     return textrect
 
 
-def getModCase(s, mod):
-    """Checks the state of the shift and caps lock keys, and switches the case of the s string if needed."""
-    if bool(mod & pygame.locals.KMOD_RSHIFT or mod & pygame.locals.KMOD_LSHIFT) ^ bool(mod & pygame.locals.KMOD_CAPS):
-        return s.swapcase()
-    else:
-        return s
-
-
 def inputMode(prompt, screenSurf, x, y, fgcol, bgcol, maxlen=12, allowed=None, pos='left', cursor='_', cursorBlink=False):
     """Takes control of the program when called. This function displays a prompt on the screen (the "prompt" string)
     parameter) on the screenSurf surface at the x, y coordinates. The text is in the fgcol color with a bgcol color
@@ -180,12 +161,17 @@ def inputMode(prompt, screenSurf, x, y, fgcol, bgcol, maxlen=12, allowed=None, p
             cursorTimestamp = time.time()
 
         for event in pygame.event.get():
-#            print event # used mainly to monitor key values for the N900 hack
             if event.type == pygame.locals.QUIT:
                 terminate()
-            if event.type == pygame.locals.KEYDOWN:
+            elif event.type == pygame.locals.MOUSEBUTTONDOWN: # Use Screen Tap as Enter Key
+                done = True
+                if cursorShow:
+                    cursorShow = '   '
+            elif event.type == pygame.locals.KEYDOWN:
                 if event.key == pygame.locals.K_ESCAPE:
-                    return None
+                    terminate()
+                elif event.key == pygame.locals.K_q and event.mod & pygame.locals.KMOD_CTRL:
+                    terminate()
                 elif event.scancode == 36: # Enter key pressed
                     done = True
                     if cursorShow:
@@ -198,19 +184,102 @@ def inputMode(prompt, screenSurf, x, y, fgcol, bgcol, maxlen=12, allowed=None, p
                     if len(inputText) >= maxlen or (allowed is not None and event.unicode not in allowed):
                         continue
                     if event.key >= 32 and event.key < 128 and event.unicode not in ('0123456789'):
-                        inputText += getModCase(chr(event.key), event.mod)
+                        inputText += pygame_utils.toProperCase(chr(event.key), event.mod)
                     elif event.unicode in ('0123456789'):
-                        inputText += getModCase(event.unicode, event.mod)
+                        inputText += pygame_utils.toProperCase(event.unicode, event.mod)
     return inputText
 
 
-def nextBananaShape(orient):
-    """Returns the next banana shape in the sequence of 0, 1, 2, 3, then 0 again. (These correspond to the RIGHT, UP,
-    LEFT, and DOWN variables."""
-    if orient + 1 == 4:
-        return 0
-    else:
-        return orient + 1
+_NUMBER_MAPPINGS = {
+    pygame.locals.K_PERIOD: ".",
+    pygame.locals.K_KP_PERIOD: ".",
+    pygame.locals.K_q: "1",
+    pygame.locals.K_w: "2",
+    pygame.locals.K_e: "3",
+    pygame.locals.K_r: "4",
+    pygame.locals.K_t: "5",
+    pygame.locals.K_y: "6",
+    pygame.locals.K_u: "7",
+    pygame.locals.K_i: "8",
+    pygame.locals.K_o: "9",
+    pygame.locals.K_p: "0",
+}
+
+
+def inputModeNum(prompt, screenSurf, x, y, fgcol, bgcol, maxlen=12, pos='left', cursor='_', cursorBlink=False):
+    """Takes control of the program when called. This function displays a prompt on the screen (the "prompt" string)
+    parameter) on the screenSurf surface at the x, y coordinates. The text is in the fgcol color with a bgcol color
+    background. You can optionally specify maxlen for a maximum length of the user's response. "allowed" is a string
+    of allowed characters (if the player can only type numbers, say) and ignores all other keystrokes. The "pos"
+    parameter can either be the string "left" (where the x, y coordinates refer to the top left corner of the text
+    box) or "center" (where the x, y coordinates refer to the top center of the text box).
+
+    "cursor" is an optional character that is used for a cursor to show where the next letter will go. If "cursorBlink"
+    is True, then this cursor character will blink on and off.
+
+    The returned value is a string of what the player typed in, or None if the player pressed the Esc key.
+
+    Note that the player can only press Backspace to delete characters, they cannot use the arrow keys to move the
+    cursor."""
+    inputText = ''
+    """inputText will store the text of what the player has typed in so far."""
+    done = False
+    cursorTimestamp = time.time()
+    cursorShow = cursor
+    while not done:
+        """We will keep looping until the player has pressed the Esc or Enter key."""
+
+        textrect = drawText(prompt + cursorShow, screenSurf, x, y, fgcol, bgcol, pos)
+        drawText(prompt + inputText + cursorShow, screenSurf, textrect.left, textrect.top, fgcol, bgcol, 'left')
+        pygame.display.update()
+        GAME_CLOCK.tick(FPS)
+
+        if cursor and cursorBlink and time.time() - 1.0 > cursorTimestamp:
+            if cursorShow == cursor:
+                cursorShow = '   '
+            else:
+                cursorShow = cursor
+            cursorTimestamp = time.time()
+
+        for event in pygame.event.get():
+            if event.type == pygame.locals.QUIT:
+                terminate()
+            elif event.type == pygame.locals.MOUSEBUTTONDOWN: # Use Screen Tap as Enter Key
+                done = True
+                if cursorShow:
+                    cursorShow = '   '
+            elif event.type == pygame.locals.KEYDOWN:
+                if event.key == pygame.locals.K_ESCAPE:
+                    terminate()
+                elif event.key == pygame.locals.K_q and event.mod & pygame.locals.KMOD_CTRL:
+                    terminate()
+                elif event.scancode == 36 or event.key in (pygame.locals.K_RETURN, pygame.locals.K_KP_ENTER):
+                    done = True
+                    if cursorShow:
+                        cursorShow = '   '
+                elif event.key == pygame.locals.K_BACKSPACE:
+                    if len(inputText):
+                        drawText(prompt + inputText + cursorShow, screenSurf, textrect.left, textrect.top, bgcol, bgcol, 'left')
+                        inputText = inputText[:-1]
+                else:
+                    if len(inputText) >= maxlen:
+                        continue
+                    if event.unicode in ('0123456789'):
+                        inputText += pygame_utils.toProperCase(event.unicode, event.mod)
+                    else:
+                        translatedChar = _NUMBER_MAPPINGS.get(event.key, None)
+                        if translatedChar is not None:
+                            inputText += pygame_utils.toProperCase(translatedChar, event.mod)
+    return inputText
+
+
+def nextBananaOrientation(orient):
+    return {
+        RIGHT: UP,
+        UP: LEFT,
+        LEFT: DOWN,
+        DOWN: RIGHT,
+    }[orient]
 
 
 def drawBanana(screenSurf, orient, x, y):
@@ -262,21 +331,21 @@ def makeCityScape():
     """We will choose an upward, downward, valley "v" curve, or hilly "^" curve for the slope of the buildings.
     Half of the time we will choose the valley slope shape, while the remaining three each have a 1/6 chance of
     being choosen. The slope also determines the height of the first building, which is stored in newHeight."""
-    slope = random.randint(1, 6)
-    if slope == 1:
+    slopeIndex = random.randint(1, 6)
+    if slopeIndex == 1:
         slope = 'upward'
         newHeight = 15
-    elif slope == 2:
+    elif slopeIndex == 2:
         slope = 'downward'
         newHeight = 130
-    elif slope >= 3 and slope <= 5:
+    elif 3 <= slopeIndex and slopeIndex <= 5:
         slope = 'v'
         newHeight = 15
     else:
         slope = '^'
         newHeight = 130
 
-    bottomLine = 455 # the bottom line of the buildings. We want some space for the wind arrow to go
+    bottomLine = SCR_HEIGHT - 25 # the bottom line of the buildings. We want some space for the wind arrow to go
     heightInc = 10 # a baseline for how much buildings grow or shrink compared to the last building
     defBuildWidth = 37 # default building width, also judges how wide buildings can be
     randomHeightDiff = 120 # about how much buildings grow or shrink
@@ -389,6 +458,8 @@ def checkForKeyPress():
         if event.type == pygame.locals.KEYUP:
             if event.key == pygame.locals.K_ESCAPE: # pressing escape quits
                 terminate()
+            elif event.key == pygame.locals.K_q and event.mod & pygame.locals.KMOD_CTRL:
+                terminate()
             return event.key
         if event.type == pygame.locals.MOUSEBUTTONDOWN:
             return True
@@ -494,14 +565,14 @@ def showSettingsScreen(screenSurf):
         p2name = 'Player 2'
 
     while points is None:
-        points = inputMode("Play to how many total points (Default = 3)?  ", screenSurf, SCR_WIDTH / 2 - 155, 110, GRAY_COLOR, BLACK_COLOR, maxlen=6, allowed='0123456789', pos='left', cursorBlink=True)
+        points = inputModeNum("Play to how many total points (Default = 3)?  ", screenSurf, SCR_WIDTH / 2 - 155, 110, GRAY_COLOR, BLACK_COLOR, maxlen=6, pos='left', cursorBlink=True)
     if points == '':
         points = 3
     else:
-        points = int(points)
+        points = int(float(points))
 
     while gravity is None:
-        gravity = inputMode("Gravity in Meters/Sec (Earth = 9.8)?  ", screenSurf, SCR_WIDTH / 2 - 150, 140, GRAY_COLOR, BLACK_COLOR, maxlen=6, allowed='0123456789.', pos='left', cursorBlink=True)
+        gravity = inputModeNum("Gravity in Meters/Sec (Earth = 9.8)?  ", screenSurf, SCR_WIDTH / 2 - 150, 140, GRAY_COLOR, BLACK_COLOR, maxlen=6, pos='left', cursorBlink=True)
     if gravity == '':
         gravity = 9.8
     else:
@@ -510,14 +581,9 @@ def showSettingsScreen(screenSurf):
     drawText('--------------', screenSurf, SCR_WIDTH / 2 -10, 170, GRAY_COLOR, BLACK_COLOR, pos='center')
     drawText('V = View Intro', screenSurf, SCR_WIDTH / 2 -10, 200, GRAY_COLOR, BLACK_COLOR, pos='center')
     drawText('P = Play Game', screenSurf, SCR_WIDTH / 2 -10, 230, GRAY_COLOR, BLACK_COLOR, pos='center')
-    #drawText('Your Choice?', screenSurf, SCR_WIDTH / 2 -10, 260, GRAY_COLOR, BLACK_COLOR, pos='center')
+    drawText('Ctrl Q = Quit', screenSurf, SCR_WIDTH / 2 -10, 260, GRAY_COLOR, BLACK_COLOR, pos='center')
     pygame.display.update()
 
-#    key = waitForPlayerToPressKey()
-#    while chr(key) != 'v' and chr(key) != 'p':
-#        key = waitForPlayerToPressKey()
-# The chr(key) function above does not work for the N900, use the following instead
-
     while choice is None:
         choice = inputMode("Your Choice?  ", screenSurf, SCR_WIDTH / 2 - 55, 260, GRAY_COLOR, BLACK_COLOR, maxlen=1, allowed='vp', pos='left', cursorBlink=True)
 
@@ -559,6 +625,7 @@ def showIntroScreen(screenSurf, p1name, p2name):
         pygame.display.update()
 
         time.sleep(0.3)
+    pygame.event.clear() # Clear the queue since the user might have been confused
 
 
 def getShot(screenSurf, p1name, p2name, playerNum):
@@ -574,23 +641,23 @@ def getShot(screenSurf, p1name, p2name, playerNum):
     else:
         x = SCR_WIDTH-100
 
-    angle = ''
-    while angle == '':
-        angle = inputMode('Angle:  ', screenSurf, x, 18, WHITE_COLOR, SKY_COLOR, maxlen=3, allowed='0123456789')
-    if angle is None:
+    angleInput = ''
+    while angleInput == '':
+        angleInput = inputModeNum('Angle:  ', screenSurf, x, 18, WHITE_COLOR, SKY_COLOR, maxlen=3)
+    if angleInput is None:
         terminate()
-    angle = int(angle)
+    angle = int(float(angleInput))
 
-    velocity = ''
-    while velocity == '':
-        velocity = inputMode('Velocity:  ', screenSurf, x, 34, WHITE_COLOR, SKY_COLOR, maxlen=3, allowed='0123456789')
-    if velocity is None:
+    velocityInput = ''
+    while velocityInput == '':
+        velocityInput = inputModeNum('Velocity:  ', screenSurf, x, 34, WHITE_COLOR, SKY_COLOR, maxlen=3)
+    if velocityInput is None:
         terminate()
-    velocity = int(velocity)
+    velocity = int(float(velocityInput))
 
     # Erase the user's input
-    drawText('Angle:   ' + str(angle), screenSurf, x, 2, SKY_COLOR, SKY_COLOR)
-    drawText('Velocity:   ' + str(angle), screenSurf, x, 2, SKY_COLOR, SKY_COLOR)
+    drawText('Angle:   %s ' % angleInput, screenSurf, x, 18, SKY_COLOR, SKY_COLOR)
+    drawText('Velocity:   %s ' % velocityInput, screenSurf, x, 34, SKY_COLOR, SKY_COLOR)
     pygame.display.update()
 
     if playerNum == 2:
@@ -599,7 +666,7 @@ def getShot(screenSurf, p1name, p2name, playerNum):
     return (angle, velocity)
 
 
-def displayScore(screenSurf, oneScore, twoScore):
+def drawScore(screenSurf, oneScore, twoScore):
     """Draws the score on the screenSurf surface."""
     scoreMessage = str(oneScore) + '>Score<' + str(twoScore)
     drawText(scoreMessage, screenSurf, SCR_WIDTH / 2, SCR_HEIGHT - 20, WHITE_COLOR, SKY_COLOR, pos='center')
@@ -635,12 +702,12 @@ def plotShot(screenSurf, skylineSurf, angle, velocity, playerNum, wind, gravity,
     pygame.display.update()
     """Draw the gorilla throwing the banana."""
 
-    bananaShape = UP
+    bananaOrient = UP
 
     if playerNum == 2:
         startx += GOR_DOWN_SURF.get_size()[0]
 
-    starty -= getBananaRect(0, 0, bananaShape).height + BAN_UP_SURF.get_size()[1]
+    starty -= getBananaRect(0, 0, bananaOrient).height + BAN_UP_SURF.get_size()[1]
 
     impact = False
     bananaInPlay = True
@@ -656,21 +723,21 @@ def plotShot(screenSurf, skylineSurf, angle, velocity, playerNum, wind, gravity,
         if x >= SCR_WIDTH - 10 or x <= 3 or y >= SCR_HEIGHT:
             bananaInPlay = False
 
-        bananaRect = getBananaRect(x, y, bananaShape)
-        if bananaShape == UP:
+        bananaRect = getBananaRect(x, y, bananaOrient)
+        if bananaOrient == UP:
             bananaSurf = BAN_UP_SURF
             bananaRect.left -= 2
             bananaRect.top += 2
-        elif bananaShape == DOWN:
+        elif bananaOrient == DOWN:
             bananaSurf = BAN_DOWN_SURF
             bananaRect.left -= 2
             bananaRect.top += 2
-        elif bananaShape == LEFT:
+        elif bananaOrient == LEFT:
             bananaSurf = BAN_LEFT_SURF
-        elif bananaShape == RIGHT:
+        elif bananaOrient == RIGHT:
             bananaSurf = BAN_RIGHT_SURF
 
-        bananaShape = nextBananaShape(bananaShape)
+        bananaOrient = nextBananaOrientation(bananaOrient)
 
         srcPixArray = pygame.PixelArray(skylineSurf)
         if bananaInPlay and y > 0:
@@ -746,14 +813,14 @@ def collideWithNonColor(pixArr, surfObj, rect, color):
     return False
 
 
-def getBananaRect(x, y, shape):
-    if shape == UP:
+def getBananaRect(x, y, orient):
+    if orient == UP:
         return pygame.Rect((x, y), BAN_UP_SURF.get_size())
-    if shape == DOWN:
+    if orient == DOWN:
         return pygame.Rect((x, y), BAN_DOWN_SURF.get_size())
-    if shape == LEFT:
+    if orient == LEFT:
         return pygame.Rect((x, y), BAN_LEFT_SURF.get_size())
-    if shape == RIGHT:
+    if orient == RIGHT:
         return pygame.Rect((x, y), BAN_RIGHT_SURF.get_size())
 
 
@@ -798,24 +865,23 @@ def doExplosion(screenSurf, skylineSurf, x, y, explosionSize=BUILD_EXPLOSION_SIZ
     pygame.display.update()
 
 
-def main():
-    pygame.mouse.set_visible(False)
-    """winSurface, being the surface object returned by pygame.display.set_mode(), will be drawn to the screen
+def game_loop():
+    """screenSurf, being the surface object returned by pygame.display.set_mode(), will be drawn to the screen
     every time pygame.display.update() is called."""
-
     # Uncomment either of the following lines to put the game into full screen mode.
-    ##winSurface = pygame.display.set_mode((SCR_WIDTH, SCR_HEIGHT), pygame.FULLSCREEN, 32)
-    winSurface = pygame.display.set_mode((SCR_WIDTH, SCR_HEIGHT), pygame.FULLSCREEN|pygame.HWSURFACE)
+    screenSurf = pygame.display.set_mode((SCR_WIDTH, SCR_HEIGHT), pygame.FULLSCREEN|pygame.HWSURFACE)
+
     ##pygame.display.toggle_fullscreen()
     pygame.display.set_caption('Gorillas.py')
+    pygame.mouse.set_visible(False)
 
-    showStartScreen(winSurface)
+    showStartScreen(screenSurf)
 
     while True:
         # start a new game
-        p1name, p2name, winPoints, gravity, nextScreen = showSettingsScreen(winSurface)
+        p1name, p2name, winPoints, gravity, nextScreen = showSettingsScreen(screenSurf)
         if nextScreen == 'v':
-            showIntroScreen(winSurface, p1name, p2name)
+            showIntroScreen(screenSurf, p1name, p2name)
 
         # Reset the score and make it the first player's turn.
         p1score = 0
@@ -826,44 +892,69 @@ def main():
         while p1score < winPoints and p2score < winPoints:
             if newRound:
                 # At the start of a new round, make a new city scape, place the gorillas, and get the wind speed.
-                skylineSurf, buildCoords = makeCityScape() # Note that the city skyline goes on skylineSurf, not winSurface.
+                skylineSurf, buildCoords = makeCityScape() # Note that the city skyline goes on skylineSurf, not screenSurf.
                 gorPos = placeGorillas(buildCoords)
                 wind = getWind()
                 newRound = False
 
             # Do all the drawing.
-            winSurface.blit(skylineSurf, (0, 0))
-            drawGorilla(winSurface, gorPos[0][0], gorPos[0][1], 0)
-            drawGorilla(winSurface, gorPos[1][0], gorPos[1][1], 0)
-            drawWind(winSurface, wind)
-            drawSun(winSurface)
-            displayScore(winSurface, p1score, p2score)
+            screenSurf.blit(skylineSurf, (0, 0))
+            drawGorilla(screenSurf, gorPos[0][0], gorPos[0][1], 0)
+            drawGorilla(screenSurf, gorPos[1][0], gorPos[1][1], 0)
+            drawWind(screenSurf, wind)
+            drawSun(screenSurf)
+            drawScore(screenSurf, p1score, p2score)
 
             pygame.display.update()
 
-            angle, velocity = getShot(winSurface, p1name, p2name, turn)
+            angle, velocity = getShot(screenSurf, p1name, p2name, turn)
             if turn == 1:
                 gorx, gory = gorPos[0][0], gorPos[0][1]
             elif turn == 2:
                 gorx, gory = gorPos[1][0], gorPos[1][1]
-            result = plotShot(winSurface, skylineSurf, angle, velocity, turn, wind, 9.8, gorPos[0], gorPos[1])
+            result = plotShot(screenSurf, skylineSurf, angle, velocity, turn, wind, 9.8, gorPos[0], gorPos[1])
 
             if result == 'gorilla1':
-                victoryDance(winSurface, gorPos[1][0], gorPos[1][1])
+                victoryDance(screenSurf, gorPos[1][0], gorPos[1][1])
                 p2score += 1
                 newRound = True
             elif result == 'gorilla2':
-                victoryDance(winSurface, gorPos[0][0], gorPos[0][1])
+                victoryDance(screenSurf, gorPos[0][0], gorPos[0][1])
                 p1score += 1
                 newRound = True
 
-            if turn == 1:
-                turn = 2
-            else:
-                turn = 1
+            turn = {
+                1: 2,
+                2: 1,
+            }[turn]
+
+        pygame.event.clear() # clears event queue, otherwise Game Over Screen does not come up
+        showGameOverScreen(screenSurf, p1name, p1score, p2name, p2score)
+
+def main():
+    try:
+        os.makedirs(constants._data_path_)
+    except OSError, e:
+        if e.errno != 17:
+            raise
+
+    logFormat = '(%(relativeCreated)5d) %(levelname)-5s %(threadName)s.%(name)s.%(funcName)s: %(message)s'
+    logging.basicConfig(level=logging.DEBUG, format=logFormat)
+    rotating = logging.handlers.RotatingFileHandler(constants._user_logpath_, maxBytes=512*1024, backupCount=1)
+    rotating.setFormatter(logging.Formatter(logFormat))
+    root = logging.getLogger()
+    root.addHandler(rotating)
+    _moduleLogger.info("%s %s-%s" % (constants.__app_name__, constants.__version__, constants.__build__))
+    _moduleLogger.info("OS: %s" % (os.uname()[0], ))
+    _moduleLogger.info("Kernel: %s (%s) for %s" % os.uname()[2:])
+    _moduleLogger.info("Hostname: %s" % os.uname()[1])
+
+    try:
+        game_loop()
+    except:
+        _moduleLogger.exception("Bailing out")
+
 
-        showGameOverScreen(winSurface, p1name, p1score, p2name, p2score)
 
 if __name__ == '__main__':
-    logging.basicConfig(level=logging.DEBUG)
     main()
